#include <Servo.h>
#include <LiquidCrystal_I2C.h>

#define RED 10
#define BLUE 2
#define GREEN 11

#define RED_P 5
#define GREEN_P 6
#define BLUE_P 4

#define BUZZER 13
#define MOTION_SENSOR 12

// servo
Servo myservo; 
volatile int pos = 0;

// senzor miscare
int motion;
boolean timer_on = false;

// timpi pentru leduri
unsigned long currentTime = 0;
unsigned long previousTime = 0;
unsigned long interval = 90000; // pragul de timp dupa care se actioneaza iar semaforul
int firstTime;

// lcd
LiquidCrystal_I2C lcd(0x27, 16, 2);

void setup()
{  
  Serial.begin(9600);
  
  pinMode(RED, OUTPUT);
  pinMode(BLUE, OUTPUT);
  pinMode(GREEN, OUTPUT);
  
  pinMode(RED_P, OUTPUT);
  pinMode(BLUE_P, OUTPUT);
  pinMode(GREEN_P, OUTPUT);

  pinMode(BUZZER, OUTPUT);
  
  pinMode(MOTION_SENSOR, INPUT_PULLUP);
  motion = 0;
  firstTime = 1;
  
  // servo
  myservo.attach(7);
  myservo.write(pos);
  delay(15);
  
  // leduri
  driver_color(0, 255, 0); // verde masini
  pedestrian_color(255, 0, 0); // rosu pietoni
 
  // lcd
  lcd.begin(16,2);
  lcd.init();
  lcd.backlight();

  lcd.setCursor(4, 0);        
  lcd.print("ASTEAPTA");        
  lcd.setCursor(1, 1);        
  lcd.print("CULOAREA VERDE");

  wait_10sec();
}

void driver_color(int red, int green, int blue) {
    analogWrite(RED, red);
    analogWrite(BLUE, blue);
    analogWrite(GREEN, green);
}

void pedestrian_color(int red, int green, int blue) {
    analogWrite(RED_P, red);
    analogWrite(BLUE_P, blue);
    analogWrite(GREEN_P, green);
}

void wait_10sec() {
  for (int i = 0; i < 10; i++) {
    delay(1000);
  }
}

void wait_20sec() {
  for (int i = 0; i < 20; i++) {
    delay(1000);
  }
}

void wait_30sec() {
  for (int i = 0; i < 30; i++) {
    delay(1000);
  }
}

void wait_60sec() {
  for (int i = 0; i < 60; i++) {
    delay(1000);
  }
}

void blink_driver_led() {
    int count = 5;
    driver_color(0, 0, 0);
    while (count >= 0) {
      count--;
      driver_color(0, 0, 0);
      delay(200);
      driver_color(0, 255, 0);
      delay(200);
    }
}

void blink_pedestrian_led() {
    int count = 5;
    pedestrian_color(0, 0, 0);
    while (count >= 0) {
      count--;
      pedestrian_color(0, 0, 0);
      delay(200);
      pedestrian_color(0, 255, 0);
      delay(200);
    }
}

void pedestrian_crossing() {
    pedestrian_color(0, 255, 0); // verde pieton
  
    lcd.clear();
    
    lcd.setCursor(2, 0);        
    lcd.print("POTI TRAVERSA"); 
  
    for (int i = 0; i <= 30; i++) {
       if (i > 20) {
          lcd.setCursor(3, 1);
          lcd.print(0);
          lcd.setCursor(4, 1);
          lcd.print(30 - i);
       } else {
          lcd.setCursor(3, 1);
          lcd.print(30 - i);
       }
      lcd.setCursor(6, 1);
      lcd.print("SECUNDE");
      delay(1000);
    }
           
    pedestrian_color(0, 255, 0); // verde pieton
}

void end_pedestrian_crossing() {
    pedestrian_color(0, 0, 0);
    blink_pedestrian_led(); // blink verde pieton
    pedestrian_color(255, 0, 0); // rosu pieton
    
    lcd.clear();
    lcd.setCursor(4, 0);        
    lcd.print("ASTEAPTA");        
    lcd.setCursor(1, 1);        
    lcd.print("CULOAREA VERDE"); 
}

void activate_buzzer() {
  for (int i = 0; i < 3; i++) {
    tone(BUZZER, 1000);
    delay(400);
    noTone(BUZZER);
    delay(150);
  }
}

void loop()
{ 
  motion = digitalRead(MOTION_SENSOR);
  
  if (motion == 1) {
    Serial.println("Motion detected");
    currentTime = millis();
    
    if (firstTime != 1) {
        while (currentTime < previousTime + interval) {
          delay(1000);
          currentTime = millis();
        }
    }
    previousTime = millis();
    firstTime = 0;
    
    wait_10sec(); // timp de siguranta
    driver_color(0, 0, 0);
    blink_driver_led(); // blink verde
    driver_color(255, 0, 0); // rosu
    driver_color(255, 0, 0);
    delay(1000);

    activate_buzzer();
    delay(2000);
    myservo.write(pos + 90);
    delay(1000); 

    pedestrian_crossing();
    end_pedestrian_crossing();

    driver_color(0, 255, 0); // verde sofer

    activate_buzzer();
    myservo.write(pos);
    delay(100);
  } else {
    Serial.println("NO Motion detected");
  }
  delay(1000);
}
